var yaml = require('yaml');
var path = require('path');
var fs   = require('fs');

var warn = require('./warn').warn;

function createModule(context) {
	return {
		superModule : context,
		subModules  : {},
		yamlData    : null,
		directory   : null
	};
}

function readModuleDeclaration(callback, context, dirname, filename) {
	fs.readFile(filename, function (err, data) {
		if (!err) {
			try {
				var yamlData = yaml.eval(data.toString()).module;
			} catch (e) {
				warn(filename, "Invalid YAML file.");
				callback();
				return ;
			}
			var module = createModule(context);
			module.yamlData = yamlData;
			module.directory = dirname;
			context.subModules[module.yamlData.name] = module;
			browseDirectory(callback, module, dirname);
		} else {
			warn.sys(filename, err.code);
			callback();
		}
	});
}

function checkModuleDeclaration(callback, context, dirname) {
	fs.stat(dirname, function (err, stat) {
		if (!err && stat.isDirectory()) {
			var filename = path.join(dirname, 'module.yml');
			fs.stat(filename, function (err, stat) {
				if (!err && stat.isFile()) {
					readModuleDeclaration(callback, context, dirname, filename);
				} else {
					if (err && err.code !== 'ENOENT') {
						warn.sys(filename, err.code);
					} else if (!err) {
						warn(filename, 'Expected a regular file.');
					}
					callback();
				}
			});
		} else if (!err) {
			warn(dirname, 'Not a directory.');
		} else {
			warn.sys(dirname, err.code);
		}
	});
}

function browseDirectory(callback, context, dirname) {
	fs.readdir(dirname, function (err, entries) {
		if (entries.length) {
			var count = entries.length;
			var decrement = function () { if (!--count) callback(); };
			for (var t = entries.length; t--;) {
				(function (entry) {
					entry = path.join(dirname, entry);
					fs.stat(entry, function (err, stat) {
						if (!err && stat.isDirectory()) {
							checkModuleDeclaration(decrement, context, entry);
						} else {
							if (err) {
								warn.sys(entry, err.code);
							}
							decrement();
						}
					});
				})(entries[t]);
			}
		} else {
			callback();
		}
	});
}

function build(callback, dirname, root) {
	checkModuleDeclaration(function () {
		callback(root);
	}, root, dirname);
}

exports.build = function (dirname, callback) {
	return build(callback, dirname, createModule(null));
};
